// Sample driver 

#include <linux/module.h>
#include <linux/interrupt.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/cdev.h>
#include <linux/irq.h>
#include <linux/io.h>
#include <linux/moduleparam.h>
#include <linux/pci.h> 
#include <linux/uaccess.h>

#define FIOQSIZE 0x5460
#define		EXAR_READ_REG      	(FIOQSIZE + 1)
#define 	EXAR_WRITE_REG     	(FIOQSIZE + 2)

unsigned long base_address;

struct xrioctl_rw_reg {
	unsigned char reg;
	unsigned char regvalue;
};

int dscud_major=112;
unsigned char __iomem *baseaddress;
static unsigned int Read(int offset)
{
	
	return readb(baseaddress + offset);
}

static void Write(int value, int offset )
{
	
	writeb(value, baseaddress + offset);
}

long sample_ioctl(struct file *filep, unsigned int cmd, 
					unsigned long arg)
{
	
	struct xrioctl_rw_reg ioctlrwarg;

	switch(cmd)
	{
	 
		case EXAR_READ_REG:
			
			if (copy_from_user(&ioctlrwarg, (void *)arg, sizeof(ioctlrwarg)))
				return -EFAULT;
			ioctlrwarg.regvalue =Read(ioctlrwarg.reg);
			if (copy_to_user((void *)arg, &ioctlrwarg, sizeof(ioctlrwarg)))
				return -EFAULT;
			break;
		
		case EXAR_WRITE_REG:
			
			if (copy_from_user(&ioctlrwarg, (void *)arg, sizeof(ioctlrwarg)))
				return -EFAULT;
			
			Write(ioctlrwarg.regvalue,base_address+ioctlrwarg.reg); 
			break;
	}
	
	return 0;		
}

static struct file_operations sample_fops = {
	
	unlocked_ioctl:  sample_ioctl,
	
};

int  init_fun(void)
{
	
	int rc=0;
	struct pci_dev *dev = NULL;
	unsigned long base, len;

	printk("The init fun get called \n");

	rc=register_chrdev(dscud_major,"ttyXR0",&sample_fops);
	if(rc < 0)
	{
		printk("Character device not registered \n");
	}
	

	dev = pci_get_device(0x13A8,0x0354,NULL);

	if(dev == NULL)
	{
		printk("Exar Device not found \n");
		return -1;
	}
	
	
	pci_enable_device(dev);
	rc = pci_enable_device(dev);
	if (rc)
	{
		printk("pci_enable_device function failed \n");
		return rc;
	}

	base = pci_resource_start(dev, 0);

	if (pci_resource_flags(dev, 0) & IORESOURCE_MEM)
	{
		len =  pci_resource_len(dev, 0);			
		baseaddress =ioremap(base, len);		
	}

	printk("DS-MPE-SER4M driver loaded \n");
	return 0;
}

void exit_fun(void)
{
	
	printk("Exit fun called \n");
       // free_irq(board->int_value,board);
 	//printk("IRQ freed \n");
	unregister_chrdev(dscud_major, "ttyXR0");

}



MODULE_AUTHOR("Diamond");
MODULE_DESCRIPTION("sample driver");
MODULE_LICENSE("GPL");



module_init(init_fun);
module_exit(exit_fun);
